/***************************************************************************
                          GameBoard.h  -  description
                             -------------------
    begin                : Thu Dec 6 2001
    copyright            : (C) 2001 by Thorsten Scheuermann
    email                : tscheuermann@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GAMEBOARD_H
#define GAMEBOARD_H

#include <qcanvas.h>

/**
  *@author Thorsten Scheuermann
  */

class Piece;
class PieceSlot;

class GameBoard : public QCanvas
{
Q_OBJECT

public: 
	GameBoard(QObject *parent=0, const char *name=0);
	~GameBoard();
	
	int ceilingWidth();
	inline int arrayWidth();
  inline int arrayHeight();
	PieceSlot *getSlot(int x, int y);
	PieceSlot *getSlot(QPoint &pt);
	void clearMarks();
	void getOverlappingSlots(Piece *p, QList<PieceSlot> *slotList);
	
signals:
	void timer();
	void ceilingLowered();
		
public slots:
	void initialize();
	void lowerCeiling();
	
protected:
	virtual void update();
	void initializeNeighbors();
	bool isValidArrayPos(int x, int y);
	
	PieceSlot **m_pieceArray;
	QPixmap *m_tileMap;
	QCanvasText *m_nextLabel;
	
	int m_arrayWidth;
	int m_arrayHeight;
	int m_numSlots;
	
	int m_ceilingWidth;
};

inline int GameBoard::arrayWidth()
{
	return m_arrayWidth;
}

inline int GameBoard::arrayHeight()
{
	return m_arrayHeight;
}

#endif
