/***************************************************************************
                          GameSettings.h  -  description
                             -------------------
    begin                : Tue Dec 18 2001
    copyright            : (C) 2001 by Thorsten Scheuermann
    email                : tscheuermann@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GAMESETTINGS_H
#define GAMESETTINGS_H

#include <qobject.h>

/**
  *@author Thorsten Scheuermann
  */

class GameSettings : public QObject
{
	Q_OBJECT
	
public: 
	GameSettings(int barSteps, int startRows, int numBadPieces,
							 int bonusBaseScore);
	~GameSettings();

	inline int barSteps() const;
	inline int startRows() const;
	inline int badPieceCount() const;
	inline int bonusBaseScore() const;
	
signals:
	void increaseBar(int n);
	void decreaseBar(int n);

public slots:
	void setBarSteps(int steps);
	void piecesRemoved(int numDirect, int numIndirect);
	
protected:
	int m_barSteps;				// number of steps in the danger bar
	int m_startRows;			// number of rows to start out with
	int m_badPieceCount;	// number of unremovable pieces
	int m_bonusBaseScore;	// the bonus score from which the number of used pieces is subtracted
};

inline int GameSettings::barSteps() const
{
	return m_barSteps;
}

inline int GameSettings::startRows() const
{
	return m_startRows;
}

inline int GameSettings::badPieceCount() const
{
	return m_badPieceCount;
}

inline int GameSettings::bonusBaseScore() const
{
	return m_bonusBaseScore;
}

#endif
