/***************************************************************************
                          HighscoreTable.h  -  description
                             -------------------
    begin                : Sat Jan 12 2002
    copyright            : (C) 2002 by Thorsten Scheuermann
    email                : tscheuermann@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef HIGHSCORETABLE_H
#define HIGHSCORETABLE_H

#include <qvaluelist.h>
#include <qstring.h>
#include <qobject.h>

/**
  *@author Thorsten Scheuermann
  */

class Config;

class HighscoreTable : public QObject
{
	Q_OBJECT

public:
	HighscoreTable(unsigned int maxEntries, const QString &scoreName);
	~HighscoreTable();

	inline int numEntries() const;
	bool isHighscore(int score);
	void addEntry(int score, QString &name);
	int getScore(int num);
	int getLatestEntryPos() const;
	void clearLatestEntryPos();
	const QString &getName(int num);
	void readScores(Config *config);
	void writeScores(Config *config);
	void resetScores(Config *config);

signals:
	void tableChanged();

protected:
	unsigned int m_maxEntries;
	QString m_scoreName;

	struct Entry
	{
		int score;
		QString name;
	};

	QValueList<struct Entry> m_entryList;
	int m_latestEntry;
};

inline int HighscoreTable::numEntries() const
{
	return m_entryList.count();
}

inline int HighscoreTable::getLatestEntryPos() const
{
	return m_latestEntry;
}

#endif
