/***************************************************************************
                          Piece.h  -  description
                             -------------------
    begin                : Thu Dec 6 2001
    copyright            : (C) 2001 by Thorsten Scheuermann
    email                : tscheuermann@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PIECE_H
#define PIECE_H

#include <qcanvas.h>
/**
  *@author Thorsten Scheuermann
  */

class PieceFactory;
class GameBoard;

class Piece : public QCanvasSprite
{
public: 
	virtual ~Piece();

	enum State {PS_Pending, PS_Launcher, PS_Moving, PS_Placed, PS_Disappearing,
							PS_Dropping};
	
	int type() const;
	void setState(enum State s);
	void setLaunchAngle(float a);
	bool update();
		
protected:
	friend class PieceFactory;
	
	Piece(int type, QCanvasPixmapArray *pmArray, GameBoard *gameBoard);
	void updateMoving();
	bool updateDropping();
	bool updateDisappearing();
	void initializeDropping();
	void initializeDisappearing();
	
	int m_type;
	enum State m_state;
	GameBoard *m_myBoard;
	int m_disappearingFrameCounter;
};

inline int Piece::type() const
{
	return m_type;
}

#endif
