/***************************************************************************
                          PieceCounter.cpp  -  description
                             -------------------
    begin                : Mon Dec 17 2001
    copyright            : (C) 2001 by Thorsten Scheuermann
    email                : tscheuermann@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "PieceCounter.h"
#include "Piece.h"
#include <stdlib.h>

PieceCounter::PieceCounter(int numTypes)
{
	m_numTypes = numTypes;
	m_numPiecesLeft = new int[m_numTypes];
	m_typesLeft = new int[m_numTypes];
	
	reset();
}

PieceCounter::~PieceCounter()
{
	delete[] m_typesLeft;
	delete[] m_numPiecesLeft;
}

void PieceCounter::reset()
{
	int i;
	
	m_numTypesLeft = 0;
	m_totalPiecesLeft = 0;

	for(i=0; i<m_numTypes; i++)
		m_numPiecesLeft[i] = 0;
}

void PieceCounter::addPiece(Piece *p)
{
	m_numPiecesLeft[p->type()]++;
	m_totalPiecesLeft++;
	if(m_numPiecesLeft[p->type()] == 1)
		m_typesLeft[m_numTypesLeft++] = p->type();	// add the new type to the list
}

void PieceCounter::removePiece(Piece *p)
{
	if(p->type() >= m_numTypes)				// make sure it isn't a bad piece
		return;

	if(m_numPiecesLeft[p->type()] == 0)
		return;

	m_numPiecesLeft[p->type()]--;
	m_totalPiecesLeft--;
	if(m_numPiecesLeft[p->type()] == 0)
		recalcTypesLeft();

}

void PieceCounter::recalcTypesLeft()
{
	m_numTypesLeft = 0;
	for(int i=0; i<m_numTypes; i++)
	{
		if(m_numPiecesLeft[i])
			m_typesLeft[m_numTypesLeft++] = i;
	}
}

int PieceCounter::getRandomRemainingType() const
{
	return m_typesLeft[rand()%m_numTypesLeft];
}
