/***************************************************************************
                          PieceSlot.cpp  -  description
                             -------------------
    begin                : Thu Dec 6 2001
    copyright            : (C) 2001 by Thorsten Scheuermann
    email                : tscheuermann@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "PieceSlot.h"
#include "Globals.h"
#include "Piece.h"
#include "GameBoard.h"

PieceSlot::PieceSlot(GameBoard *board, int posX, int posY)
	: QObject(board)
{
	m_myBoard = board;
	m_piece = NULL;
	m_arrayPos = QPoint(posX, posY);
	m_mark = false;
	m_neighbors.setAutoDelete(false);
}

PieceSlot::~PieceSlot()
{
}
	
void PieceSlot::setPiece(Piece *p)
{
	m_piece = p;
	if(!p)
		return;
	
	QPoint sPos = screenPosition();
	p->move(sPos.x(), sPos.y());
	p->setState(Piece::PS_Placed);
}

QPoint PieceSlot::arrayPosition()
{
	return m_arrayPos;
}

QPoint PieceSlot::screenPosition()
{
	return QPoint(m_arrayPos.x()*PIECE_WIDTH + (m_arrayPos.y()%2)*PIECE_WIDTH/2,
								(m_arrayPos.y() + m_myBoard->ceilingWidth())*PIECE_HEIGHT);
}

QRect PieceSlot::boundingRect()
{
	return QRect(screenPosition(), QSize(PIECE_WIDTH, PIECE_HEIGHT));
}

float PieceSlot::squaredDist(Piece *p)
{
	float a, b;
	
	a = 4+m_arrayPos.x()*PIECE_WIDTH + (m_arrayPos.y()%2)*PIECE_WIDTH/2 - p->x();
	b = 4+(m_arrayPos.y() + m_myBoard->ceilingWidth())*PIECE_HEIGHT - p->y();
	return a*a + b*b;
}

void PieceSlot::addNeighbor(PieceSlot *p)
{
	m_neighbors.append(p);
}

QList<PieceSlot> *PieceSlot::neighborList()
{
	return &m_neighbors;
}

void PieceSlot::removeDFS(QList<PieceSlot> *l, int type)
{
	PieceSlot *s;
	
	if(isMarked())
		return;
	setMark(true);

	if(!m_piece)
		return;
	
	if(m_piece->type() != type)
		return;
	
	// I have the right piece type, so add myself to the list
	l->append(this);
	
	for(s=m_neighbors.first(); s; s=m_neighbors.next())
		s->removeDFS(l, type);
}


void PieceSlot::connectionDFS()
{
	PieceSlot *s;

	if(!m_piece)
		return;	
	if(isMarked())
		return;
	setMark(true);
	
	for(s=m_neighbors.first(); s; s=m_neighbors.next())
		s->connectionDFS();
}

void PieceSlot::updatePiecePosition()
{
	if(m_piece)
	{
		QPoint sPos = screenPosition();
		m_piece->move(sPos.x(), sPos.y());
	}
}
