/***************************************************************************
                          PieceSlot.h  -  description
                             -------------------
    begin                : Thu Dec 6 2001
    copyright            : (C) 2001 by Thorsten Scheuermann
    email                : tscheuermann@gmx.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PIECESLOT_H
#define PIECESLOT_H

#include <qpoint.h>
#include <qrect.h>
#include <qobject.h>
#include <qlist.h>

/**
  *@author Thorsten Scheuermann
  */
class Piece;
class GameBoard;

class PieceSlot : public QObject
{
public: 
	PieceSlot(GameBoard *board, int posX, int posY);
	~PieceSlot();
	
	void setPiece(Piece *p);
	inline Piece *piece();
	QPoint arrayPosition();
	QPoint screenPosition();
	QRect boundingRect();
	void updatePiecePosition();
	float squaredDist(Piece *p);
	
	inline void setMark(bool m);
	inline bool isMarked();

	void addNeighbor(PieceSlot *p);
	QList<PieceSlot> *neighborList();
	void removeDFS(QList<PieceSlot> *l, int type);
	void connectionDFS();
	
protected:
	GameBoard *m_myBoard;
	Piece *m_piece;
	QPoint m_arrayPos;
	bool m_mark;
	QList<PieceSlot> m_neighbors;
};


inline void PieceSlot::setMark(bool m)
{
	m_mark = m;
}

inline bool PieceSlot::isMarked()
{
	return m_mark;
}

inline Piece *PieceSlot::piece()
{
	return m_piece;
}

#endif
